==> doip-sdk-1.5.6-sources.jar: org/bdware/doip/endpoint/server/v3/TCPListener.java <==
/*
 *    Copyright (c) [2021] [Peking University]
 *    [BDWare DOIP SDK] is licensed under Mulan PSL v2.
 *    You can use this software according to the terms and conditions of the Mulan PSL v2.
 *    You may obtain a copy of Mulan PSL v2 at:
 *             http://license.coscl.org.cn/MulanPSL2
 *    THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT, MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 *    See the Mulan PSL v2 for more details.
 */

package org.bdware.doip.endpoint.server.v3;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.LengthFieldBasedFrameDecoder;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bdware.doip.codec.v3.DOIPCodec;
import org.bdware.doip.codec.v3.DOIPV3Printer;
import org.bdware.doip.endpoint.server.DoipListenerConfig;

public class TCPListener {

    private final int port;
    static Logger logger = LogManager.getLogger(TCPListener.class);
    private final DoipListenerConfig listenerConfig;
    private Channel ch;
    private ServerHandler handler;

    public TCPListener(int port, DoipListenerConfig listenerConfig) {
        this.port = port;
        this.listenerConfig = listenerConfig;
    }

    public void setRequestHandler(RequestHandler handler) {
        this.handler = new ServerHandler(handler, listenerConfig);
    }

    public void start() {
        if (handler == null) {
            logger.error("Handler not init yet! set handler first");
            return;
        }
        EventLoopGroup bossGroup = new NioEventLoopGroup(1);
        EventLoopGroup workerGroup = new NioEventLoopGroup();
        int maxFrame = 5 * 1024 * 1024;
        try {
            ServerBootstrap b = new ServerBootstrap();
            b.option(ChannelOption.WRITE_BUFFER_WATER_MARK,
                    new WriteBufferWaterMark(2 * maxFrame, 10 * maxFrame));
            b.group(bossGroup, workerGroup)
                    .channel(NioServerSocketChannel.class)
                    .localAddress(port);
            b.childHandler(
                    new ChannelInitializer<SocketChannel>() {
                        @Override
                        protected void initChannel(SocketChannel ch) throws Exception {
                            ChannelPipeline pipline = ch.pipeline();
                            pipline.addLast(new LengthFieldBasedFrameDecoder(maxFrame,
                                            20, 4, 0, 0))
                                    .addLast(new DOIPCodec()).addLast(new DOIPV3Printer());
                            pipline.addLast(handler);
                            //  pipline.addLast(new NaiveEnvelopeToDoMessage()).addLast(new DoipMessagePrinter());
                        }
                    });

            ch = b.bind().syncUninterruptibly().channel();
            logger.info("TCP DOIP V3 listener start at:" + port);
            ch.closeFuture().sync();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            bossGroup.shutdownGracefully();
        }
    }

    public void stop() {
        ch.close();
    }

}

==> doip-sdk-1.5.91-sources.jar: org/bdware/doip/endpoint/server/v3/TCPListener.java <==
/*
 *    Copyright (c) [2021] [Peking University]
 *    [BDWare DOIP SDK] is licensed under Mulan PSL v2.
 *    You can use this software according to the terms and conditions of the Mulan PSL v2.
 *    You may obtain a copy of Mulan PSL v2 at:
 *             http://license.coscl.org.cn/MulanPSL2
 *    THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT, MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 *    See the Mulan PSL v2 for more details.
 */

package org.bdware.doip.endpoint.server.v3;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.LengthFieldBasedFrameDecoder;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.bdware.doip.codec.v3.DOIPCodec;
import org.bdware.doip.codec.v3.DOIPV3Printer;
import org.bdware.doip.endpoint.server.DoipListenerConfig;

public class TCPListener {

    private final int port;
    static Logger logger = LogManager.getLogger(TCPListener.class);
    private final DoipListenerConfig listenerConfig;
    private Channel ch;
    private ServerHandler handler;

    public TCPListener(int port, DoipListenerConfig listenerConfig) {
        this.port = port;
        this.listenerConfig = listenerConfig;
    }

    public void setRequestHandler(RequestHandler handler) {
        this.handler = new ServerHandler(handler, listenerConfig);
    }

    public void start() {
        if (handler == null) {
            logger.error("Handler not init yet! set handler first");
            return;
        }
        EventLoopGroup bossGroup = new NioEventLoopGroup(1);
        EventLoopGroup workerGroup = new NioEventLoopGroup();
        int maxFrame = 5 * 1024 * 1024;
        try {
            ServerBootstrap b = new ServerBootstrap();
            b.option(ChannelOption.WRITE_BUFFER_WATER_MARK,
                    new WriteBufferWaterMark(2 * maxFrame, 10 * maxFrame));
            b.group(bossGroup, workerGroup)
                    .channel(NioServerSocketChannel.class)
                    .localAddress(port);
            b.childHandler(
                    new ChannelInitializer<SocketChannel>() {
                        @Override
                        protected void initChannel(SocketChannel ch) throws Exception {
                            ChannelPipeline pipline = ch.pipeline();
                            pipline.addLast(new LengthFieldBasedFrameDecoder(maxFrame,
                                            20, 4, 0, 0))
                                    .addLast(new DOIPCodec());
                            pipline.addLast(handler);
                        }
                    });

            ch = b.bind().syncUninterruptibly().channel();
            logger.info("TCP DOIP V3 listener start at:" + port);
            ch.closeFuture().sync();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            bossGroup.shutdownGracefully();
        }
    }

    public void stop() {
        ch.close();
    }

}
